﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;
using System.IO;
using System.Threading;



///////////////////////////////////////////////////////////////////////////////
//
//  www.interestprograms.ru /программы, игры и их исходные коды/
//  Протокол TCP клиентское приложение. Часть 1.
//
///////////////////////////////////////////////////////////////////////////////


namespace tcpclient
{
    public partial class TcpClientForm : Form
    {
        public TcpClientForm()
        {
            InitializeComponent();
        }

        // Объект содержащий рабочий сокет клиентского приложения
        TcpClient _tcpСlient = new TcpClient();

        // Объект сетевого потока для приема и отправки сообщений
        NetworkStream ns;

        // Флаг для остановки потоков и завершения сетевой работы приложения
        bool _stopNetwork;

        #region Управление клиентским приложением

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            Connect();
        }

        private void buttonSend_Click(object sender, EventArgs e)
        {
            SendMessage();
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            CloseClient();
        }

        #endregion


        #region Функциональная часть Сетевая работа

        // Попытка подключения к серверу
        void Connect()
        {
            try
            {
                _tcpСlient.Connect(textBoxIP.Text, int.Parse(textBoxPort.Text));

                ns = _tcpСlient.GetStream();

                Thread th = new Thread(ReceiveRun);
                th.Start();

                // Цветовое оповещение о подключении.
                BackColor = Color.FromArgb(255, 128, 0);
            }
            catch 
            {
                ErrorSound();
            }
        }

        void CloseClient()
        {
            if (ns != null) ns.Close();
            if (_tcpСlient != null) _tcpСlient.Close();

            _stopNetwork = true;

            // Цветовое оповещение об отключении.
            BackColor = Color.FromName("Control");
        }


        // Отправка сообщений в блокирующем режиме,
        // поскольку обмен короткими сообщениями
        // не вызовет заметного блокирования интерфейса приложения. 
        void SendMessage()
        {
            if (ns != null)
            {
                byte[] buffer = Encoding.Default.GetBytes(textBoxSend.Text);
                ns.Write(buffer, 0, buffer.Length);
            }
        }


        // Цикл извлечения сообщений,
        // запускается в отдельном потоке.
        void ReceiveRun()
        {
            while (true)
            {
                try
                {
                    string s = null;
                    while (ns.DataAvailable == true)
                    {
                        // Определение необходимого размера буфера приема.
                        byte[] buffer = new byte[_tcpСlient.Available];

                        ns.Read(buffer, 0, buffer.Length);
                        s += Encoding.Default.GetString(buffer);
                    }

                    if (s != null)
                    {
                        ShowReceiveMessage(s);
                        s = String.Empty;
                    }


                    // Вынужденная строчка для экономия ресурсов процессора.
                    // Неизящный способ.
                    Thread.Sleep(100);
                }
                catch
                {
                    ErrorSound();
                }

                if (_stopNetwork == true) break;

            }
        }
        #endregion


        #region Информации о сетевой работе

        // Код доступа к свойствам объектов главной формы  из других потоков
        delegate void UpdateReceiveDisplayDelegate(string message);
        void ShowReceiveMessage(string message)
        {
            if (listBox1.InvokeRequired == true)
            {
                UpdateReceiveDisplayDelegate rdd = new UpdateReceiveDisplayDelegate(ShowReceiveMessage);

                // Данный метод вызывается в дочернем потоке,
                // ищет основной поток и выполняет делегат указанный в качестве параметра 
                // в главном потоке, безопасно обновляя интерфейс формы.
                Invoke(rdd, new object[] { message });
            }
            else
            {
                // Если не требуется вызывать метод Invoke, обратимся напрямую к элементу формы.
                listBox1.Items.Add(message);
            }
        }

        // Звуковое оповещение о перехваченной ошибке.
        void ErrorSound()
        {
            Console.Beep(2000, 80);
            Console.Beep(3000, 120);
        }

        #endregion
    }
}
